
`timescale 1 ns / 1 ps

module picosoc_demo (
    input  clk_in,
    input  resetn,
    output trap,
	
	input  irq_5,
	input  irq_6,
	input  irq_7,
	
	output ser_tx,
	input  ser_rx
	
);
	parameter [0:0] BARREL_SHIFTER = 0;
	parameter [0:0] ENABLE_MULDIV = 0;
	parameter [0:0] ENABLE_COMPRESSED = 0;
	parameter [0:0] ENABLE_COUNTERS = 1;
	parameter [0:0] ENABLE_IRQ_QREGS = 0;

	parameter integer MEM_WORDS = 256;
	parameter [31:0] STACKADDR = (4*MEM_WORDS);       // end of memory, (4*MEM_WORDS)
	parameter [31:0] PROGADDR_RESET = 32'h 0000_0400; // 
	parameter [31:0] PROGADDR_IRQ = 32'h 0000_0400;

	reg [31:0] irq;
	wire irq_stall = 0;
	wire irq_uart = 0;

    wire clk;

	always @* begin
		irq = 0;
		irq[3] = irq_stall;
		irq[4] = irq_uart;
		irq[5] = 1'b0;//irq_5;
		irq[6] = 1'b0;//irq_6;
		irq[7] = 1'b0;//irq_7;
	end

    wire mem_valid;
    wire mem_instr;
    reg  mem_ready;

    wire [31:0] mem_addr;
    wire [31:0] mem_wdata;
    wire [3:0]  mem_wstrb;
    reg  [31:0] mem_rdata;
	

	wire        simpleuart_reg_div_sel;
	wire [31:0] simpleuart_reg_div_do;
	
	wire        simpleuart_reg_dat_sel;
	wire [31:0] simpleuart_reg_dat_do;
	wire        simpleuart_reg_dat_wait;

	wire        smem_ready;
    wire [31:0] smem_rdata;
	
	wire        smem_valid;
	wire [3:0]  smem_wstrb;
	wire [31:0] smem_addr;
	wire [31:0] smem_wdata;
	
    reg ram_ready;
    wire [31:0] ram_rdata;

	assign smem_valid = mem_valid;//mem_valid && (mem_addr[31:24] > 8'h00);
	assign smem_wstrb = mem_wstrb;
	assign smem_addr  = mem_addr - 32'h400;
	assign smem_wdata = mem_wdata;

	assign mem_ready = (smem_valid && smem_ready) || 
                                        ram_ready ||
                           simpleuart_reg_div_sel || 
                       (simpleuart_reg_dat_sel && !simpleuart_reg_dat_wait);

	assign mem_rdata = (smem_valid && smem_ready) ? smem_rdata : 
                                        ram_ready ? ram_rdata :
                           simpleuart_reg_div_sel ? simpleuart_reg_div_do :
                           simpleuart_reg_dat_sel ? simpleuart_reg_dat_do : 
                                                    32'h 0000_0000;

    assign simpleuart_reg_div_sel = mem_valid && (mem_addr == 32'h 0200_0004);
    assign simpleuart_reg_dat_sel = mem_valid && (mem_addr == 32'h 0200_0008);

    GW_PLL pll_12_5MHz (
        .clkout(clk), 
        .clkin (clk_in)
    );

	picorv32 #(
		.STACKADDR(STACKADDR),
		.PROGADDR_RESET(PROGADDR_RESET),
		.PROGADDR_IRQ(PROGADDR_IRQ),
		.BARREL_SHIFTER(BARREL_SHIFTER),
		.COMPRESSED_ISA(ENABLE_COMPRESSED),
		.ENABLE_COUNTERS(ENABLE_COUNTERS),
		.ENABLE_MUL(ENABLE_MULDIV),
		.ENABLE_DIV(ENABLE_MULDIV),
		.ENABLE_IRQ(1),
		.ENABLE_IRQ_QREGS(ENABLE_IRQ_QREGS)
		) core (
		.clk         (clk        ),
		.resetn      (resetn     ),
		.trap        (trap       ),
		.mem_valid   (mem_valid  ),
		.mem_instr   (mem_instr  ),
		.mem_ready   (mem_ready  ),
		.mem_addr    (mem_addr   ),
		.mem_wdata   (mem_wdata  ),
		.mem_wstrb   (mem_wstrb  ),
		.mem_rdata   (mem_rdata  ),
		.irq         (irq        )
	);

    picorv32_smem smem (
        .clk         (clk         ),
        .mem_valid   (smem_valid && (mem_addr >= 4*MEM_WORDS) && (mem_addr < 32'h 0200_0000)), 
        .mem_ready   (smem_ready  ), 
        .mem_addr    (smem_addr[11:0]), 
        .mem_wdata   (smem_wdata  ),  
        .mem_wstrb   (smem_wstrb  ), 
        .mem_rdata   (smem_rdata  )
    );

	always @(posedge clk)
		ram_ready <= mem_valid && !mem_ready && mem_addr < 4*MEM_WORDS;

	picosoc_mem #(.WORDS(MEM_WORDS)) memory (
		.clk(clk),
		.wen((mem_valid && !mem_ready && mem_addr < 4*MEM_WORDS) ? mem_wstrb : 4'b0),
		.addr(mem_addr[23:2]),
		.wdata(mem_wdata),
		.rdata(ram_rdata)
    );

	
	simpleuart simpleuart (
		.clk         (clk         ),
		.resetn      (resetn      ),

		.ser_tx      (ser_tx      ),
		.ser_rx      (ser_rx      ),

		.reg_div_we  (simpleuart_reg_div_sel ? mem_wstrb : 4'b 0000),
		.reg_div_di  (mem_wdata),
		.reg_div_do  (simpleuart_reg_div_do),

		.reg_dat_we  (simpleuart_reg_dat_sel ? mem_wstrb[0] : 1'b0),
		.reg_dat_re  (simpleuart_reg_dat_sel && !mem_wstrb),
		.reg_dat_di  (mem_wdata),
		.reg_dat_do  (simpleuart_reg_dat_do),
		.reg_dat_wait(simpleuart_reg_dat_wait)
	);


endmodule  


module picosoc_mem #(
	parameter integer WORDS = 256
) (
	input clk,
	input [3:0] wen,
	input [21:0] addr,
	input [31:0] wdata,
	output reg [31:0] rdata
);
	reg [31:0] mem [0:WORDS-1] /*synthesis syn_ramstyle = "block_ram"*/;

	always @(posedge clk) begin
		rdata <= mem[addr];
		if (wen[0]) mem[addr][ 7: 0] <= wdata[ 7: 0];
		if (wen[1]) mem[addr][15: 8] <= wdata[15: 8];
		if (wen[2]) mem[addr][23:16] <= wdata[23:16];
		if (wen[3]) mem[addr][31:24] <= wdata[31:24];
	end
endmodule



